// Programming Techniques - Project
// Ken Webb
// January 5, 2004
// Application.cpp: implementation of the Application class.
//
//////////////////////////////////////////////////////////////////////

#include "Application.h"
#include "Msg.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Application::Application()
{
	factory = NULL;
	inherHier = NULL;
	containHier = NULL;
	root = NULL;
	becRoot = NULL;
	maxProcessLoops = 10; // default
}

Application::~Application()
{
	delete containHier;
	delete inherHier;
	delete factory;
	// don't delete root
}

///////////////////
// Initialize
///////////////////
void Application::initialize( char *configFileName )
{
	// Read values from configuration file
	readConfigFromFile( configFileName );

	// Factory
	factory = new TreeNodeFactory( maxBioEntities, maxBioEntityClasses );

	// BioEntityClass
	BioEntityClass::seedRandomNumberGenerator();
	BioEntityClass::setBevalRangeValues(bevalGeneMin, bevalGeneMax, bevalPheneMin, bevalPheneMax);

	// Inheritance Hierarchy
	inherHier = new InheritanceHierarchy;
	inherHier->setFactory( factory );
	if (Msg::infoM) {
		printf("\nStep 1: Identify entities, inheritance and containment hierarchies\n");
		printf("\n1.1: Identify entities in the problem domain (biology) ...\n");
	}
	inherHier->populateNamesFromFile( bioEntityClassNamesFile );
	if (Msg::infoM) inherHier->printBioEntityClassNames();
	if (Msg::infoM)
		printf("\n1.2: Identify inheritance relations between these entities ...\n");
	inherHier->populateFromFile( inheritanceHierarchyDataFile );

	// BioEntityClass root
	becRoot = inherHier->getRootNode();
	if (Msg::infoM) becRoot->preOrderPrint();

	// Containment Hierarchy
	containHier = new ContainmentHierarchy;
	containHier->setFactory( factory );
	containHier->setInheritanceHierarchy( inherHier );
	if (Msg::infoM)
		printf("\n1.3: Identify containment relations between these entities ...\n");
	containHier->populateFromFile( containmentHierarchyDataFile );

	// BioEntity root
	root = containHier->getRootNode();
	if (Msg::infoM) root->preOrderPrint();
	if (Msg::infoM)
		printf("\nStep 2: Establish adjacency and other relationships between entities ...\n");
	root->configure(); // recursive configure
	// if (Msg::debugM) printf( "Application::initialize(): done.\n" );
}

///////////////////
// Process
///////////////////
void Application::process( void )
{
	if (Msg::appM) printf( "\n" );
	for (int j = 0; j < maxProcessLoops; j++) {
		root->act();
		if (Msg::appM) printf( "\n" );
	}
}

/////////////////////
// Wrap up
/////////////////////
void Application::wrapup( void )
{
	// nothing needs to be done
	if (Msg::debugM) printf( "perfCount = %u\n", BioEntity::getPerfCount() );
}

////////////////////////////////////////////
// Read configuration values from a file
////////////////////////////////////////////
void Application::readConfigFromFile( char *fileName )
{
	FILE *fp;
	char *mode = "r";
	const int sizeBuff = 100;
	char buffer[sizeBuff];
	int intVal;
	bool showParms = false;

	if ((fp = fopen(fileName, mode)) == NULL) {
		if (Msg::errorM) printf( "Can't open file: %s in %s mode\n", fileName, mode);
		exit(0);
	}
	else {
		// printf("Opened file: %s in %s mode\n", fileName, mode);
		if (fgets(buffer, sizeBuff, fp) != NULL) {
			if (sscanf( buffer, "%d", &intVal ) != 1) {
				if (Msg::errorM) printf("Application: error reading from %s\n", fileName);
				exit(0);
			}
			if (intVal == 1)
				showParms = true;
		}
		if (fgets(buffer, sizeBuff, fp) != NULL) {
			if (sscanf( buffer, "%d %d %d", &Msg::appM, &Msg::infoM, &Msg::errorM ) != 3) {
				if (Msg::errorM) printf("Application: error reading from %s\n", fileName);
				exit(0);
			}
			if (showParms) {
				printf( "Msg::appM   : %d\n", Msg::appM );
				printf( "Msg::infoM  : %d\n", Msg::infoM );
				printf( "Msg::errorM : %d\n", Msg::errorM );
			}
		}
		if (fgets(buffer, sizeBuff, fp) != NULL) {
			if (sscanf( buffer, "%d %d", &maxBioEntities, &maxBioEntityClasses ) != 2) {
				if (Msg::errorM) printf("Application: error reading from %s\n", fileName);
				exit(0);
			}
			if (showParms) {
				printf( "maxBioEntities : %d\n", maxBioEntities );
				printf( "maxBioEntityClasses : %d\n", maxBioEntityClasses );
			}
		}
		if (fgets(buffer, sizeBuff, fp) != NULL) {
			if (sscanf( buffer, "%d %d %d %d", &bevalGeneMin, &bevalGeneMax, &bevalPheneMin, &bevalPheneMax ) != 4) {
				if (Msg::errorM) printf("Application: error reading from %s\n", fileName);
				exit(0);
			}
			if (showParms) {
				printf( "bevalGeneMin : %d\n", bevalGeneMin );
				printf( "bevalGeneMax : %d\n", bevalGeneMax );
				printf( "bevalPheneMin : %d\n", bevalPheneMin );
				printf( "bevalPheneMax : %d\n", bevalPheneMax );
			}
		}
		if (fgets(buffer, sizeBuff, fp) != NULL) {
			if (sscanf( buffer, "%d", &maxProcessLoops ) != 1) {
				if (Msg::errorM) printf("Application: error reading from %s\n", fileName);
				exit(0);
			}
			if (showParms)
				printf( "maxProcessLoops : %d\n", maxProcessLoops );
		}
		if (fgets(buffer, sizeBuff, fp) != NULL) {
			if (sscanf( buffer, "%s ", bioEntityClassNamesFile ) != 1) {
				if (Msg::errorM) printf("Application: error reading from %s\n", fileName);
				exit(0);
			}
			if (showParms)
				printf( "bioEntityClassNamesFile : %s\n", bioEntityClassNamesFile );
		}
		if (fgets(buffer, sizeBuff, fp) != NULL) {
			if (sscanf( buffer, "%s ", inheritanceHierarchyDataFile ) != 1) {
				if (Msg::errorM) printf("Application: error reading from %s\n", fileName);
				exit(0);
			}
			if (showParms)
				printf( "inheritanceHierarchyDataFile : %s\n", inheritanceHierarchyDataFile );
		}
		if (fgets(buffer, sizeBuff, fp) != NULL) {
			if (sscanf( buffer, "%s ", containmentHierarchyDataFile ) != 1 ) {
				if (Msg::errorM) printf("Application: error reading from %s\n", fileName);
				exit(0);
			}
			if (showParms)
				printf( "containmentHierarchyDataFile : %s\n", containmentHierarchyDataFile );
		}
		fclose(fp);
	}
}
