// Programming Techniques - Project
// Ken Webb
// January 6, 2004
// BioEntity.cpp: implementation of the BioEntity class.
//
//////////////////////////////////////////////////////////////////////

#include "BioEntity.h"
#include "Msg.h"
#include <string.h>
#include <ctype.h>

//////////////////////////////////////////
// Class variable initializations
//////////////////////////////////////////
char* BioEntity::nameBuffer = new char[MaxNameBuffSize];
unsigned long BioEntity::perfCount = 0;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

BioEntity::BioEntity()
{
	// initialize all instance variables to safe values
	id = 0;
	becId = 0;
	beClass = NULL;
	beType = Betypexxxxxxxxx;
	geneVal = 0;
	pheneVal = 0;
	pSmInt = NULL;
	pSmExt = NULL;
}

BioEntity::~BioEntity()
{
}

//////////////////////////////////////////////
// getId - get id of this BioEntity instance
//////////////////////////////////////////////
int BioEntity::getId( void )
{ return id; }

//////////////////////////////////////////////////////////////////////////
// getName - get globally unique name of this BioEntity instance
// Returned value must be immediately copied to more permanent storage
//////////////////////////////////////////////////////////////////////////
char* BioEntity::getName( void )
{
	if (beClass == NULL)
		sprintf( nameBuffer, "%s_%d", "UnknownClassName", id );
	else {
		sprintf( nameBuffer, "%s_%d", beClass->getName(), id );
		// Convention: first letter of enities are lower case, classes upper case
		nameBuffer[0] = tolower(nameBuffer[0]);
	}
	return nameBuffer;
}

///////////////
// setId
///////////////
void BioEntity::setId( int entityId )
{ id = entityId; }

/////////////////////////////////////////
// getBecId - get BioEntityClass Id
/////////////////////////////////////////
int BioEntity::getBecId( void )
{ return becId; }

////////////////////////////////////////////////
// getBecName - get the BioEntityClass name
////////////////////////////////////////////////
char* BioEntity::getBecName( void )
{
	if (beClass == NULL)
		return "UnknownClassName";
	else
		return beClass->getName();
}

/////////////////////////////////////////////////////////////
// setBec - set BioEntityClass Id and pointer to the class
/////////////////////////////////////////////////////////////
void BioEntity::setBec( int classId, BioEntityClass* becNode )
{
	if (becNode == NULL) {
		if (Msg::errorM) {
			printf( "BioEntity::setBec: becNode is NULL\n");
			exit(0);
		}
	}
	becId = classId;
	beClass = becNode;
}

////////////////////////////////////////////
// getBeType, getGeneVal, getPheneVal
////////////////////////////////////////////
int BioEntity::getBeType( void )
{ return beType; }
int BioEntity::getGeneVal( void )
{ return geneVal; }
int BioEntity::getPheneVal( void )
{ return pheneVal; }

//////////////////////////////////////////////////////////////////////////
// incPheneVal - increment or decrement the phene value by incAmount
//////////////////////////////////////////////////////////////////////////
void BioEntity::incPheneVal( int incAmount )
{ pheneVal += incAmount; }

////////////////////////////////////////////////////////////
// configure - do something once at the start
// recursive; application should call this only for root
////////////////////////////////////////////////////////////
void BioEntity::configure( void )
{
	// printf( "beClass = %x\n", beClass );
	if (beClass == NULL) {
		if (Msg::errorM) printf("BioEntity::configure: beClass is NULL\n");
		exit(0);
	}
	char* instructions = beClass->getConfigurationInstructions();
	if (Msg::infoM) printf( "%s: %s ", getName(), instructions );
	// carry out the instructions
	char state = 'x'; // initial state
	BioEntity* node = this;
	int len = strlen(instructions);
	int i = 0;
	char inChar = '\0';
	while (i < len) {
		// printf( " node: %s", node->getName() );
		switch( *(instructions + i) ) {
		case 'i': // internal
			if (state == 'e') // finished parsing the external location
				pSmExt = node;
			state = 'i';
			node = this;
			i++;
			break;
		case 'e': // internal
			if (state == 'i') // finished parsing the internal location
				pSmInt = node;
			state = 'e';
			node = this;
			i++;
			break;
		case 'U': // up
			if (node != NULL)
				node = (BioEntity*)node->getParent();
			i++;
			break;
		case 'D': // down
			if (node != NULL)
				node = (BioEntity*)node->getLeftChild();
			i++;
			break;
		case 'L': // left
			// not yet implemented
			// if (node != NULL)
			//	node = (BioEntity*)node->getLeftSibling();
			i++;
			break;
		case 'R': // right
			if (node != NULL)
				node = (BioEntity*)node->getRightSibling();
			i++;
			break;
		case '.': // stop
			if (state == 'e') // finished parsing the external location
				pSmExt = node;
			else if (state == 'i') // finished parsing the internal location
				pSmInt = node;
			state = 'x';
			i++;
			break;
		case 't': // type
			i++;
			beType = *(instructions + i);
			i++;
			break;
		case 'g': // gene
			i++;
			geneVal = atoi( instructions + i );
			if (Msg::infoM) printf( "[BioEntity geneVal: %d]", geneVal );
			// increment i so it's after the n-digit number
			inChar = *(instructions + i);
			while (isdigit((int)inChar) || inChar == '-') {
				i++;
				inChar = *(instructions + i);
			}
			break;
		case 'p': // phene
			i++;
			pheneVal = atoi( instructions + i );
			if (Msg::infoM) printf( "[BioEntity pheneVal: %d]", pheneVal );
			// increment i so it's after the n-digit number
			inChar = *(instructions + i);
			while (isdigit((int)inChar) || inChar == '-') {
				i++;
				inChar = *(instructions + i);
			}
			break;
		default:
			if (Msg::errorM) printf( "BioEntity configure: error\n" );
			i++;
			break;
		}
	}

	// Print informational details
	if (Msg::infoM) {
		if (pSmInt != NULL)
			printf( "pSmInt: %s ", pSmInt->getName() );
		if (pSmExt != NULL)
			printf( "pSmExt: %s ", pSmExt->getName() );
		printf( "\n" );
	}

	// Print header line for application data
	if (Msg::appM && !Msg::infoM) {
		switch( becId ) {
		// Pure Passive Objects
		case CytosolTDE:
		case ExtraCellularSolutionTDE:
		case MitochondrialIntermembranesolTDE:
		case MatrixsolTDE:
			printf( "%s,", getName() );
			break;
		// Container and other objects have nothing to do
		default:
			break;
		}
	}

	// execute recursively
	if (leftChild != NULL)
		leftChild->configure();
	if (rightSibling != NULL)
		rightSibling->configure();
}

//////////////////////////////////////////////////////////////////////////
// act - do something characteristic of this class of BioEntity
// recursive; application should call this only for root
// should be checking to ensure the phene values can't become negative
//////////////////////////////////////////////////////////////////////////
void BioEntity::act( void )
{
	if (Msg::debugM) perfCount++;

	switch( becId ) {

	// Pure Active Objects
	case EnzymeTDE:
		if (pSmInt != NULL)
			pSmInt->incPheneVal( geneVal );
		break;
	case CellBilayerTDE:
	case MitochondrialOuterBilayerTDE:
	case MitochondrialInnerBilayerTDE:
	case PyruvateTransporterTDE:
		if ((pSmInt != NULL) && (pSmExt != NULL)) {
			pSmInt->incPheneVal( geneVal ); // inc
			pSmExt->incPheneVal( geneVal * -1 ); // dec
		}
		break;
	// Pure Passive Objects
	case CytosolTDE:
	case ExtraCellularSolutionTDE:
	case MitochondrialIntermembranesolTDE:
	case MatrixsolTDE:
		// if (Msg::appM) printf( "[%s: %d]", getName(), pheneVal );
		if (Msg::appM) printf( "%d,", pheneVal );
		break;
	// Container and other objects have nothing to do
	default:
		break;
	}

	// execute recursively
	if (leftChild != NULL)
		leftChild->act();
	if (rightSibling != NULL)
		rightSibling->act();
}

////////////////////////////////////
// Traversal functions - not used
////////////////////////////////////
void BioEntity::preOrder( void ) {}
void BioEntity::inOrder( void ) {}
void BioEntity::postOrder( void ) {}

// tree traversal print functions that MAY be overridden in concrete subclasses

////////////////////////////////////////////////////////////////////
// preOrderPrint
// Print current node; then traverse children and siblings
////////////////////////////////////////////////////////////////////
void BioEntity::preOrderPrint( int level )
{
	printNode( level );
	if (leftChild != NULL)
		leftChild->preOrderPrint( level + 1);
	if (rightSibling != NULL)
		rightSibling->preOrderPrint( level );
}

/////////////////////////////////////////////////////////////////////////
// inOrderPrint
// Traverse children; then print current node; then traverse siblings
/////////////////////////////////////////////////////////////////////////
void BioEntity::inOrderPrint( int level )
{
	if (leftChild != NULL)
		leftChild->inOrderPrint( level + 1 );
	printNode( level );
	if (rightSibling != NULL)
		rightSibling->inOrderPrint( level );
}

////////////////////////////////////////////////////////////////////
// postOrderPrint
// Traverse children and siblings; then print current node
////////////////////////////////////////////////////////////////////
void BioEntity::postOrderPrint( int level )
{
	if (leftChild != NULL)
		leftChild->postOrderPrint( level );
	if (rightSibling != NULL)
		rightSibling->postOrderPrint( level + 1 );
	printNode( level );
}

/////////////////////////////////////////////////////////////////
// printNode - called by the various xOrderPrint() routines
/////////////////////////////////////////////////////////////////
void BioEntity::printNode( int level )
{
	while (level > 0) {
		printf( "%c", '.' );
		level--;
	}
	if (beClass == NULL)
		printf( "%s", "UnknownClassName" );
	else
		printf( "%s", getName() );
	printf( "\n" );
}

///////////////////////////////////////////////////////////////////
// getPerfCount - get performance counter for debug purposes
///////////////////////////////////////////////////////////////////
unsigned long BioEntity::getPerfCount( void )
{ return perfCount; }

