// Programming Techniques - Project
// Ken Webb
// January 5, 2004
// BioEntity.h: interface for the BioEntity class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_BIOENTITY_H__BB1BED49_2D28_4301_BB17_F448563631D1__INCLUDED_)
#define AFX_BIOENTITY_H__BB1BED49_2D28_4301_BB17_F448563631D1__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/**************************************************************************************
                                        BioEntity

A BioEntity is a concrete subclass of TreeNode.
It represents some biological entity,
and is typically found as a node in a ContainmentHierarchy tree.
It encapsulates:
  a unique (not enforced) id.
  the id of and a pointer to an associated BioEntityClass (becId, beClass).
  and five values that it gets from its BioEntityClass:
    a BioEntity type (beType).
    four associated optional values:
      geneVal  - single value associated with a gene and an active object.
      pheneVal - single value associated with a phenotype instance in a passive object.
      pSmInt   - pointer from an active object to an internal passive object.
      pSmExt   - pointer from an active object to an external passive object.

Each BioEntity object goes through an initial configure() process,
and may subsequently go through a regular act() process at each time step.

At configure time:
If its beType = BetypePureActiveObject Then
  It will have a gene value (geneVal) that it gets from BioEntityClass.
  It will search for adjacent internal (pSmInt) and/or external (pSmExt) spaces that
  contain phene values, using instructions from BioEntityClass.
Else if its beType = beTypePurePassiveObject Then
  It will have a phene value (pheneVal) that it gets from BioEntityClass.
  It will probably be acted upon by one or more betypePureActiveObject instances
  at each time step.
Else if its beType = betypePureContainer Then
  It is only a container for objects of the other two beTypes.
Endif

At act time:
If its beType = BetypePureActiveObject Then
  If it's an Enzyme Then
    Increment or decrement the internal (pSmInt) pheneVal it points to,
	by an amount corresponding to its geneVal.
  Else if it's a Bilayer or a TransportProtein Then
    Move geneVal number of units between the internal (pSmInt) pheneVal it points to,
	and the external (pSmExt) pheneVal it points to.
  Endif
Else if its beType = beTypePurePassiveObject Then
  Print the current value of its pheneVal.
Endif

An Enzyme active object is a simple abstraction of a biological enzyme that acts on small
molecules (Sm) contained within the same internal space.
A bilayer is a simple abstraction of a biological membrane (lipid bilayer) that moves small
molecules between its internal space and the space external to it.
A transport protein is a simple abstraction that acts much like a bilayer.

See also BioEntityClass.h
**************************************************************************************/

#include "TreeNode.h"
#include "BioEntityClass.h"

// Maximum size of the name buffer
const int MaxNameBuffSize = 100;

class BioEntity : public TreeNode  
{
public:
	// constructor and destructor
	BioEntity();
	virtual ~BioEntity();

	// get and set functions
	int getId( void );          // id
	void setId( int entityId );
	char* getName( void );      // name (a constructed value)
	int getBecId( void );       // BioEntityClass id
	char* getBecName( void );
	void setBec( int classId, BioEntityClass* becNode ); // set becId, and ptr to becNode
	int getBeType( void );      // beType
	int getGeneVal( void );     // geneVal
	int getPheneVal( void );    // pheneVal

	// incPheneVal - increment pheneVal by a given amount (+ or -)
	void incPheneVal( int incAmount );

	// configure - do something once
	void configure( void );

	// act - do something characteristic of this class of BioEntity
	void act( void );

	// getPerfCount - used to measure time performance
	static unsigned long getPerfCount( void );

	// tree traversal functions that MUST be overridden in concrete subclasses
	// allow concrete instances to perform some application specific function
	virtual void preOrder( void );
	virtual void inOrder( void );
	virtual void postOrder( void );

	// tree traversal print functions that MAY be overridden in concrete subclasses
	virtual void preOrderPrint( int level = 0 );
	virtual void inOrderPrint( int level = 0 );
	virtual void postOrderPrint( int level = 0 );

private:
	// private data members
	int id; // id number of this entity
	int becId; // id of class associated with this entity
	int beType; // type as specified by BioEntityClass
	int geneVal; // Gene value contained within an active object
	int pheneVal; // Phene value contained within a passive object
	BioEntityClass *beClass; // class associated with this entity
	BioEntity *pSmInt; // pointer to a small molecule space/solution located internally
	BioEntity *pSmExt; // pointer to a small molecule space/solution located externally
	static char* nameBuffer; // only one needed for the whole class
	static unsigned long perfCount; // how many times act() function is called

	// private member function
	void printNode( int level ); // called by preOrderPrint(), etc.
};

#endif // !defined(AFX_BIOENTITY_H__BB1BED49_2D28_4301_BB17_F448563631D1__INCLUDED_)
