// Programming Techniques - Project
// Ken Webb
// January 5, 2004
// BioEntityClass.cpp: implementation of the BioEntityClass class.
//
//////////////////////////////////////////////////////////////////////

#include "BioEntityClass.h"
#include "Msg.h"
#include <string.h>
#include <time.h>

//////////////////////////////////////
// Initialize class variables
//////////////////////////////////////
char* BioEntityClass::configBuffer = new char[MaxConfigBuffSize];
// set default values
int BioEntityClass::BevalGeneMin  =   -100;
int BioEntityClass::BevalGeneMax  =    100;
int BioEntityClass::BevalPheneMin =  10000;
int BioEntityClass::BevalPheneMax = 100000;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

BioEntityClass::BioEntityClass()
{
	id = 0;
	name = NULL;
}

BioEntityClass::~BioEntityClass()
{
}

///////////////////////////////////////////////////////
// getId - get id of this BioEntityClass instance
///////////////////////////////////////////////////////
int BioEntityClass::getId( void )
{ return id; }

///////////////
// setId
///////////////
void BioEntityClass::setId( int classId )
{ id = classId; }

////////////////
// getName
////////////////
char* BioEntityClass::getName( void )
{ return name; }

/////////////////
// setName
/////////////////
void BioEntityClass::setName( char* className )
{ name = className; }

///////////////////////////////////////////////////////
// setBevalRangeValues - set Beval range values
///////////////////////////////////////////////////////
void BioEntityClass::setBevalRangeValues( int gMin, int gMax, int pMin, int pMax )
{
	BevalGeneMin = gMin;
	BevalGeneMax = gMax;
	BevalPheneMin = pMin;
	BevalPheneMax = pMax;
}

/////////////////////////////////////////////////////////////////////
// getConfigurationInstructions - get configuration instructions
// Each character is an instruction to be followed:
//   U Go up one level to parent
//   D Go down one level to leftChild
//   L Go left to leftSibling
//   R Go right to rightSibling
//   i Start looking for some internal space/solution
//   e Start looking for some external space/solution
//   . Stop parsing
//   t BioEntity Type, followed by a single digit
//   g Gene, follwed by an n digit gene value
//   p Phene, followed by an n digit phene value
// ex: "iUD.t4g-538" for an Enzyme active object
//     "t2p1234567" for a Cytosol passive object
/////////////////////////////////////////////////////////////////////
char* BioEntityClass::getConfigurationInstructions( void )
{
	switch( id ) {

	// Pure Active Objects (t4)
	case EnzymeTDE:
		return(createConfigInfo("iUD.", BetypePureActiveObject, BevalGene));
		break;
	case CellBilayerTDE:
		return(createConfigInfo("eUUUiURD.", BetypePureActiveObject, BevalGene));
		break;
	case MitochondrialOuterBilayerTDE:
		return(createConfigInfo("eUUUUDiURRD.", BetypePureActiveObject, BevalGene));
		break;
	case MitochondrialInnerBilayerTDE:
		return(createConfigInfo("eURDiUURD.", BetypePureActiveObject, BevalGene));
		break;
	case PyruvateTransporterTDE:
		return(createConfigInfo("eURDiUURD.", BetypePureActiveObject, BevalGene));
		break;

	default:
		// Pure Passive Objects
		if (parent == NULL) {
			if (Msg::errorM) printf("BioEntityClass::getConfigurationInstructions: parent is NULL\n");
			exit(0);
		}
		else if (((BioEntityClass*)parent)->getId() == CellularSolutionTDE)
			return(createConfigInfo(NULL, BetypePurePassiveObject, BevalPhene));
		// Pure Containers (t1)
		else
			return(createConfigInfo(NULL, BetypePureContainer, BevalNone));
		break;
	}
}

////////////////////////////////////////////////////////////////////////////////
// createConfigInfo
// Create a configuration string to be returned to a BioEntity instance
// The returned value is volatile and must be immediately copied or used.
////////////////////////////////////////////////////////////////////////////////
char* BioEntityClass::createConfigInfo( char* navInfo, const char beType, const char geneOrPhene )
{
	int rVal = 0;
	if (geneOrPhene == BevalGene)
		rVal = getRandomInt( BevalGeneMin, BevalGeneMax );
	else if (geneOrPhene == BevalPhene)
		rVal = getRandomInt( BevalPheneMin, BevalPheneMax );

	switch (beType) {
	case BetypePureActiveObject:
		sprintf( configBuffer, "%st%c%c%d", navInfo, beType, geneOrPhene, rVal );
		break;
	case BetypePurePassiveObject:
		sprintf( configBuffer, "t%c%c%d", beType, geneOrPhene, rVal );
		break;
	case BetypePureContainer:
		sprintf( configBuffer, "t%c", beType );
		break;
	default:
		break;
	}
	return configBuffer;
}

///////////////////////////////////////////////////
// getRandomInt
// Get random integer >= minVal and < maxVal
///////////////////////////////////////////////////
int BioEntityClass::getRandomInt( int minVal, int maxVal )
{
	float rVal = (float)(rand() - 1) / (float)RAND_MAX;
	int rNum = (int)(rVal * (maxVal - minVal)) + minVal;
	return rNum;
}

///////////////////////////////////////////
// seedRandomNumberGenerator - seed the random number generator
///////////////////////////////////////////
void BioEntityClass::seedRandomNumberGenerator( void )
{
	int seed;
	long ltime;
	time( &ltime );
	seed = ltime;
	srand( seed );
	// get an initial random number, to get around a compiler bug
	rand();
}

////////////////////////////////////
// Traversal functions - not used
////////////////////////////////////
void BioEntityClass::preOrder( void ) {}
void BioEntityClass::inOrder( void ) {}
void BioEntityClass::postOrder( void ) {}

// tree traversal print functions that MAY be overridden in concrete subclasses

////////////////////////////////////////////////////////////////////
// preOrderPrint
// Print current node; then traverse children and siblings
////////////////////////////////////////////////////////////////////
void BioEntityClass::preOrderPrint( int level )
{
	printNode( level );
	if (leftChild != NULL)
		leftChild->preOrderPrint( level + 1 );
	if (rightSibling != NULL)
		rightSibling->preOrderPrint( level );
}

/////////////////////////////////////////////////////////////////////////
// inOrderPrint
// Traverse children; then print current node; then traverse siblings
/////////////////////////////////////////////////////////////////////////
void BioEntityClass::inOrderPrint( int level )
{
	if (leftChild != NULL)
		leftChild->inOrderPrint( level + 1 );
	printNode( level );
	if (rightSibling != NULL)
		rightSibling->inOrderPrint( level );
}

////////////////////////////////////////////////////////////////////
// postOrderPrint
// Traverse children and siblings; then print current node
////////////////////////////////////////////////////////////////////
void BioEntityClass::postOrderPrint( int level )
{
	if (leftChild != NULL)
		leftChild->postOrderPrint( level + 1 );
	if (rightSibling != NULL)
		rightSibling->postOrderPrint( level );
	printNode( level );
}

////////////////////////////////////////////////////
// Called by the various xOrderPrint() routines
////////////////////////////////////////////////////
void BioEntityClass::printNode( int level )
{
	while (level > 0) {
		printf( "%c", '.' );
		level--;
	}
	printf( "%s", getName() );
	printf( "\n" );
	// printf( "[%d,%d,%s] ", level, id, name );
}
