// Programming Techniques - Project
// Ken Webb
// January 5, 2004
// BioEntityClass.h: interface for the BioEntityClass class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_BIOENTITYCLASS_H__BCAC3670_B693_4277_9058_23A43D42F23C__INCLUDED_)
#define AFX_BIOENTITYCLASS_H__BCAC3670_B693_4277_9058_23A43D42F23C__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/**************************************************************************************
                                        BioEntityClass

A BioEntityClass is a concrete subclass of TreeNode.
It represents a type of biological entity,
and is typically found as a node in an InheritanceHierarchy tree.
It encapsulates a unique (not enforced) id, and a name.
The primary role of a BioEntityClass instance is to provide configuration instructions
for BioEntity instances of its type.
All BioEntity instances should call getConfigurationInstructions(), and parse the
returned string. See BioEntityClass.cpp for details on each character in that string.

See also BioEntity.h
**************************************************************************************/

#include "TreeNode.h"
#include "TypeData.h"

// BioEntity Types (see Webb & White, SAC 2004 paper)
// There are three attributes that make up a BioEntity type:
//   Beh whether or not the BioEntity has Behavior
//   Fgs whether or not the BioEntity has Fine-grained Structure
//   Con whether or not the BioEntity is a container for other BioEntities
const char Betypexxxxxxxxx = '0';
const char BetypexxxxxxCon = '1'; // Pure Container
const char BetypexxxFgsxxx = '2'; // Pure Passive Object
const char BetypexxxFgsCon = '3';
const char BetypeBehxxxxxx = '4'; // Pure Active Object
const char BetypeBehxxxCon = '5';
const char BetypeBehFgsxxx = '6';
const char BetypeBehFgsCon = '7';
// Aliases for the three types actually used in this system
const char BetypePureContainer     = BetypexxxxxxCon;
const char BetypePurePassiveObject = BetypexxxFgsxxx;
const char BetypePureActiveObject  = BetypeBehxxxxxx;

// Gene or Phene
// Active objects get a Gene value
// Passive objects get a Phene value
const char BevalGene  = 'g';
const char BevalPhene = 'p';
const char BevalNone  = 'n';

// Maximum size of the configuration buffer
const int MaxConfigBuffSize = 100;

class BioEntityClass : public TreeNode  
{
public:
	// Constructors and Destructors
	BioEntityClass();
	virtual ~BioEntityClass();

	// get and set functions
	int getId( void );                 // id
	void setId( int classId );
	char* getName( void );             // name
	void setName( char* className );
	// Set Beval range values
	static void setBevalRangeValues( int gMin, int gMax, int pMin, int pMax );

	// Get configuration instructions
	char* getConfigurationInstructions( void );

	// Application program must call this function once
	static void seedRandomNumberGenerator( void );

	// tree traversal functions that MUST be overridden in concrete subclasses
	// allow concrete instances to perform some application specific function
	virtual void preOrder( void );
	virtual void inOrder( void );
	virtual void postOrder( void );

	// tree traversal print functions that MAY be overridden in concrete subclasses
	virtual void preOrderPrint( int level = 0 );
	virtual void inOrderPrint( int level = 0 );
	virtual void postOrderPrint( int level = 0 );

private:
	// private data members
	int id; // id number of this class
	char *name; // name of the class
	static char* configBuffer; // only one needed for the whole class
	// Min and Max range values for Gene and Phene; class constants
	static int BevalGeneMin;  // any valid integer, negative, positive or 0
	static int BevalGeneMax;  // any valid integer >= BevalGeneMin
	static int BevalPheneMin; // any valid integer >= 0
	static int BevalPheneMax; // any valid integer >= BevalPheneMin

	// private member functions
	// Return a string containing configuration info
	char* createConfigInfo( char* navInfo, const char beType, const char geneOrPhene );

	// Get a random integer >= minVal and < maxVal
	int getRandomInt( int minVal, int maxVal );

	// Does actual printf for other functions
	void printNode( int level );
};

#endif // !defined(AFX_BIOENTITYCLASS_H__BCAC3670_B693_4277_9058_23A43D42F23C__INCLUDED_)
