// Programming Techniques - Project
// Ken Webb
// January 6, 2004
// InheritanceHierarchy.cpp: implementation of the InheritanceHierarchy class.
//
//////////////////////////////////////////////////////////////////////

#include "InheritanceHierarchy.h"
#include "TypeData.h"
#include "Msg.h"
#include <string.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

InheritanceHierarchy::InheritanceHierarchy()
{
	rootNode = NULL;
	factory = NULL;
	for (int i = 0; i < MaxBeClasses; i++) {
		className[i] = NULL;
		classPtr[i]  = NULL;
	}
}

InheritanceHierarchy::~InheritanceHierarchy()
{
	if (rootNode != NULL) {
		// Should return all nodes to factory using post order recursive
		// descent starting with root.
	}
	for (int i = 0; i < MaxBeClasses; i++) {
		if (className[i] != NULL)
			delete className[i];
	}
}

////////////////////////////////////////////////////
// setFactory - set singleton TreeNodeFactory
////////////////////////////////////////////////////
void InheritanceHierarchy::setFactory( TreeNodeFactory* f )
{ factory = f; }

///////////////////////////////////////////////////////////////////////
// getClassNode - get pointer to a specified BioEntityClass node
///////////////////////////////////////////////////////////////////////
BioEntityClass* InheritanceHierarchy::getClassNode( int beClassId )
{ return classPtr[beClassId]; }

/////////////////////////////////////////////////////////////////
// getRootNode - get root node of the inheritance hierarchy
/////////////////////////////////////////////////////////////////
BioEntityClass* InheritanceHierarchy::getRootNode( void )
{ return rootNode; }

////////////////////////////////////////////////////////////////////////
// populateFromFile - populate the tree by reading items from a file
////////////////////////////////////////////////////////////////////////
void InheritanceHierarchy::populateFromFile( char *fileName )
{
	BioEntityClass *oldNode = NULL;
	BioEntityClass *newNode = NULL;
	int oldLevel = 0;
	int newLevel = 0;
	int becId = 0; // BioEntityClass id from file

	FILE *fp = openFile( fileName );

	// handle root node
	int rc = getData( fp, &oldLevel, &becId );
	if (rc != EOF) {
		rootNode = factory->getBioEntityClassNode();
		oldNode = rootNode;
		oldNode->setId( becId );
		oldNode->setName( className[becId] );
		classPtr[becId] = rootNode;
	}

	// handle non-root nodes
	rc = getData( fp, &newLevel, &becId );
	while (rc != EOF) {
		newNode = factory->getBioEntityClassNode();
		newNode->setId( becId );
		newNode->setName( className[becId] );
		classPtr[becId] = newNode;
		if (newLevel > oldLevel) { // oldNode is parent of newNode
			newNode->setParentChildLinks( oldNode );
		}
		else {
			while (newLevel < oldLevel) { // have reached end of sibling chain
				oldLevel--;
				oldNode = (BioEntityClass*)oldNode->getParent();
			}
			newNode->setParentSiblingLinks( oldNode );
		}
		oldLevel = newLevel;
		oldNode = newNode;
		rc = getData( fp, &newLevel, &becId );
	}

	closeFile( fp );
}

////////////////////////////////////////////////////////////////
// populateNamesFromFile - populate names array from file
////////////////////////////////////////////////////////////////
void InheritanceHierarchy::populateNamesFromFile( char *fileName )
{
	const int SizeBuff = 100;
	char buffer[SizeBuff];
	char *name;
	int ix = 0;
	int len = 0;

	FILE *fp = openFile( fileName );

	for (int i = 0; i < 4; i++) // throw away first 4 lines
		fgets(buffer, SizeBuff, fp);

	while ((len = getName( fp, buffer, SizeBuff )) != EOF) {
		if (buffer[0] == '}') // throw away last 3 lines
			break;
		name = new char[len+1];
		strcpy( name, buffer );
		className[ix] = name;
		ix++;
	}

	closeFile( fp );
}

////////////////////////////////////
// openFile - open input file
////////////////////////////////////
FILE* InheritanceHierarchy::openFile( char *fileName )
{
	FILE* fp = fopen( fileName, "r" );
	if (fp == NULL) {
		if (Msg::errorM) printf( "Can't open file %s\n", fileName );
		exit (0);
	}
	return fp;
}

///////////////////////////////
// closeFile - close file
///////////////////////////////
void InheritanceHierarchy::closeFile( FILE *fp )
{
	fclose( fp );
}

///////////////////////////////////////////////////////////////////////
// getData - get level and id in inheritence hierarchy (from file)
///////////////////////////////////////////////////////////////////////
int InheritanceHierarchy::getData( FILE *fp, int *level, int *id )
{
	int rc = 1; // a value other than EOF
	const int SizeBuff = 100;
	static char buffer[SizeBuff];
	if (fgets(buffer, SizeBuff, fp) != NULL) {
		if (sscanf( buffer, "%d,%d", level, id ) != 2) {
			if (Msg::errorM)
				printf("InheritanceHierarchy: error reading from file\n");
			exit(0);
		}
	}
	else {
		rc = EOF;
	}
	return rc;
}

//////////////////////////////////////////////////////////
// getName - get a BioEntityClass name (from file)
// ex: EnzymeTDE, --> Enzyme
//////////////////////////////////////////////////////////
int InheritanceHierarchy::getName( FILE *fp, char *buffer, int sizeBuff )
{
	int len = 1; // a value other than EOF
	if (fgets(buffer, sizeBuff, fp) != NULL) {
		len = strlen( buffer ) - 5;
		*(buffer + len) = '\0';
	}
	else {
		len = EOF;
	}
	return len;
}

////////////////////////////////////////////////////////////////////////
// printBioEntityClassNames - print out the BioEntityClass names
////////////////////////////////////////////////////////////////////////
void InheritanceHierarchy::printBioEntityClassNames( void )
{
	int i;
	for (i = 0; i < MaxBeClasses; i++) {
		if (className[i] == NULL) // no need to go any further
			break;
		printf( "%3d %s\n", i, className[i] );
		if (i == EnzymeTDE)
			printf( "Enzyme: %d %s\n", i, className[i] );
	}
	printf( "There are %d BioEntityClass names.\n", i );
}
