// Programming Techniques - Project
// Ken Webb
// January 5, 2004
// TreeNodeFactory.cpp: implementation of the TreeNodeFactory class.
//
//////////////////////////////////////////////////////////////////////

#include "TreeNodeFactory.h"
#include "Msg.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

//////////////////////////
// Static Allocation    //
//////////////////////////
#if ALLOC_TYPE == ALLOC_STATIC

// Need to handle Q_FULL condition in constructors, although it shouldn't occur
// default constructor; uses default array and queue sizes
TreeNodeFactory::TreeNodeFactory()
{
	initialize( MaxBioEntities_Default, MaxBioEntityClasses_Default );
}

// preferred constructor
TreeNodeFactory::TreeNodeFactory( int maxBioEntities, int maxBioEntityClasses )
{
	initialize( maxBioEntities, maxBioEntityClasses );
}

TreeNodeFactory::~TreeNodeFactory()
{
	delete bioEntityQ;
	delete bioEntityClassQ;
}

//////////////////////////////////////////////////////////////////////
// getBioEntityNode - get a BioEntity instance from the factory
// Need to handle Q_EMPTY condition gracefully
//////////////////////////////////////////////////////////////////////
BioEntity* TreeNodeFactory::getBioEntityNode( void )
{
	BioEntity* bEnt = (BioEntity *)bioEntityQ->dequeue();
	if (bEnt == Q_EMPTY) {
		if (Msg::errorM) printf( "TreeNodeFactory::getBioEntityNode: Q_EMPTY" );
		exit(1);
	}
	bEnt->initialize(); // set all links to NULL
	return bEnt;
}

/////////////////////////////////////////////////////////////////////////////
// getBioEntityClassNode - get a BioEntityClass instance from the factory
// Need to handle Q_EMPTY condition gracefully
/////////////////////////////////////////////////////////////////////////////
BioEntityClass* TreeNodeFactory::getBioEntityClassNode( void )
{
	BioEntityClass* bEnt = (BioEntityClass *)bioEntityClassQ->dequeue();
	if (bEnt == Q_EMPTY) {
		if (Msg::errorM) printf( "TreeNodeFactory::getBioEntityClassNode: Q_EMPTY" );
		exit(1);
	}
	bEnt->initialize(); // set all links to NULL
	return bEnt;
}

///////////////////////////////////////////////////////////////////////
// returnBioEntityNode - return a BioEntity instance to the factory
// Need to handle Q_FULL condition
///////////////////////////////////////////////////////////////////////
void TreeNodeFactory::returnBioEntityNode( BioEntity* node )
{
	if (bioEntityQ->enqueue( (void *)node ) == Q_FULL) {
		if (Msg::errorM) printf( "TreeNodeFactory::returnBioEntityNode: Q_FULL" );
		exit(1);
	}
}

/////////////////////////////////////////////////////////////////////////////////
// returnBioEntityClassNode - return a BioEntityClass instance to the factory
// Need to handle Q_FULL condition
/////////////////////////////////////////////////////////////////////////////////
void TreeNodeFactory::returnBioEntityClassNode( BioEntityClass* node )
{
	if (bioEntityClassQ->enqueue( (void *)node ) == Q_FULL) {
		if (Msg::errorM) printf( "TreeNodeFactory::returnBioEntityClassNode: Q_FULL" );
		exit(1);
	}
}

//////////////////////////////////////////////////
// initialize - initialize arrays and queues
//////////////////////////////////////////////////
void TreeNodeFactory::initialize( int maxBioEntities, int maxBioEntityClasses )
{
	int i;
	bioEntityQ = new Queue( maxBioEntities );
	bioEntityClassQ = new Queue( maxBioEntityClasses );
	bioEntity = new BioEntity[maxBioEntities];
	for (i = 0; i < maxBioEntities; i++)
		bioEntityQ->enqueue( (void *)(&bioEntity[i]) );
	bioEntityClass = new BioEntityClass[maxBioEntityClasses];
	for (i = 0; i < maxBioEntityClasses; i++)
		bioEntityClassQ->enqueue( (void *)(&bioEntityClass[i]) );
	if (Msg::infoM) printf("Using static allocation in arrays of TreeNode instances\n");
	if (Msg::debugM) printf("BioEntity current queue size: %u\n", bioEntityQ->getSize());
	if (Msg::debugM) printf("BioEntityClass current queue size: %u\n", bioEntityClassQ->getSize());
}

#endif

////////////////////////////////////////////////////////
// Dynamic Allocation - not yet fully implemented     //
////////////////////////////////////////////////////////
#if ALLOC_TYPE == ALLOC_DYNAMIC

TreeNodeFactory::TreeNodeFactory()
{
	printf( "Using dynamic allocation (new) of TreeNode instances\n" );
}

TreeNodeFactory::TreeNodeFactory( int maxBioEntities, int maxBioEntityClasses )
{
}

TreeNodeFactory::~TreeNodeFactory()
{
}

BioEntity* TreeNodeFactory::getBioEntityNode( void )
{
	return new BioEntity;
}

BioEntityClass* TreeNodeFactory::getBioEntityClassNode( void )
{
	return new BioEntityClass;
}

void TreeNodeFactory::returnBioEntityNode( BioEntity* node )
{
	if (node != NULL)
		delete node;
}

void TreeNodeFactory::returnBioEntityClassNode( BioEntityClass* node )
{
	if (node != NULL)
		delete node;
}

void TreeNodeFactory::initialize( int maxBioEntities, int maxBioEntityClasses )
{
}

#endif
