// Programming Techniques - Project
// Ken Webb
// January 5, 2004
// TreeNodeFactory.h: interface for the TreeNodeFactory class.
//
//////////////////////////////////////////////////////////////////////

/**************************************************************************************
                                    TreeNodeFactory

Factory to get and return concrete instances of TreeNode subclasses (BioEntity,
and BioEntityClass) with all data members initialized to 0 or NULL.
This is a Singleton class; there should only be one instance in the whole system.
  See Gamma et al Patterns book for how to ensure that this really is a Singleton.
Internally, the TreeNode instances are maintained in arrays, one for each subclass.
This could be changed later if necessary, while having no effect on the
rest of the system.
The TreeNode instances can be allocated statically in arrays, or dynamically using new.
  Use the ALLOC_ #defines to control this.
  ALLOC_STATIC is the default, and is preferred in this implementation.
Advantages of array implementation:
  More flexibility, especially when debugging a large system.
  Could use the underlying array to find memory leaks (non-deleted TreeNodes).
  There should be no additional run-time overhead after initial configuration.
  In fact run-time performance is typically improved by using static rather than
  dynamic allocation, if there is high volatility.

**************************************************************************************/

#if !defined(AFX_TREENODEFACTORY_H__30EFEC7C_5D2A_484C_82CA_373996008102__INCLUDED_)
#define AFX_TREENODEFACTORY_H__30EFEC7C_5D2A_484C_82CA_373996008102__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "BioEntity.h"
#include "BioEntityClass.h"
#include "Queue.h"

// Quantity of each subclass of TreeNode maintained by factory.
// Default values that should be overridden by using the appropriate constructor.
const int MaxBioEntities_Default      = 10000;
const int MaxBioEntityClasses_Default =   500;

// Compiler directives
#define ALLOC_STATIC  1 // static allocation of TreeNode objects
#define ALLOC_DYNAMIC 2 // dynamic allocation of TreeNode objects
#define ALLOC_TYPE ALLOC_STATIC

class TreeNodeFactory  
{
public:
	// constructors and destructor
	TreeNodeFactory();
	TreeNodeFactory( int maxBioEntities, int maxBioEntityClasses );
	virtual ~TreeNodeFactory();

	// get a node from the factory
	BioEntity* getBioEntityNode( void );
	BioEntityClass* getBioEntityClassNode( void );

	// return a node to the factory
	void returnBioEntityNode( BioEntity* node );
	void returnBioEntityClassNode( BioEntityClass* node );

private:
	// Arrays to store the actual entities
	BioEntity* bioEntity;
	BioEntityClass* bioEntityClass;

	// Queues to store pointers to entities that are available for use
	Queue* bioEntityQ;
	Queue* bioEntityClassQ;

	// Common inialize function
	void initialize( int maxBioEntities, int maxBioEntityClasses );
};

#endif // !defined(AFX_TREENODEFACTORY_H__30EFEC7C_5D2A_484C_82CA_373996008102__INCLUDED_)
